#include "Game.h"
#include "CollisionDetection.h"

// 色情報を取る関数
int GetPixel(SDL_Surface *surface, int x, int y);

// プレーヤーとマップの色判定
bool CollisionDetection::PlayerAndMap(SDL_Surface *screen, int pX, int pY, int direction) {
    int i;
    int returnNo = 1;

    switch (direction) {
        case UP:
            for (i = 0; i < PLAYERWIDTH; i++) {
                if (GetPixel(screen, pX + i, pY - 1) == WALL || GetPixel(screen, pX + i, pY - 1) == BLACK) {
                    returnNo = 0;
                    break;
                }
            }
            break;
        case DOWN:
            for (i = 0; i < PLAYERWIDTH; i++) {
                if (GetPixel(screen, pX + i, pY + PLAYERHEIGHT) == WALL || GetPixel(screen, pX + i, pY + PLAYERHEIGHT) == BLACK) {
                    returnNo = 0;
                    break;
                }
            }
            break;
        case LEFT:
            for (i = 0; i < PLAYERHEIGHT; i++) {
                if (GetPixel(screen, pX - 1, pY + i) == WALL || GetPixel(screen, pX - 1, pY + i) == BLACK) {
                    returnNo = 0;
                    break;
                }
            }
            break;
        case RIGHT:
            for (i = 0; i < PLAYERHEIGHT; i++) {
                if (GetPixel(screen, pX + PLAYERWIDTH, pY + i) == WALL || GetPixel(screen, pX + PLAYERWIDTH, pY + i) == BLACK) {
                    returnNo = 0;
                    break;
                }
            }
            break;
    }
    return returnNo;
}

// プレーヤーとアイテムの位置判定
bool CollisionDetection::PlayerAndItem(int pX, int pY, int eX, int eY) {
    if (pX + 1 <= eX + ITEMSIZE && pX + PLAYERHEIGHT - 1 >= eX && pY + 1 <= eY + ITEMSIZE && pY + PLAYERHEIGHT - 1 >= eY) {
        return true;
    } else {
        return false;
    }
}

// アイテムとアイテムの位置判定
bool CollisionDetection::ItemAndItem(int iX1, int iY1, int iX2, int iY2) {
    if (iX1 < iX2 + ITEMSIZE && iX1 + ITEMSIZE > iX2 && iY1 < iY2 + ITEMSIZE && iY1 + ITEMSIZE > iY2) {
        return true;
    } else {
        return false;
    }
}

// 色情報を取る関数
int GetPixel(SDL_Surface *surface, int x, int y) {
    if (x < 0 || x > WIDTH)return 0xff;
    if (y < 0 || y > HEIGHT)return 0xff;
    int bpp = surface->format->BytesPerPixel;
    /* この p は取得したいピクセルを指すアドレス */
    Uint8 *p = (Uint8 *) surface->pixels + y * surface->pitch + x * bpp;

    switch (bpp) {
        case 1:
            return *p;
        case 2:
            return *(Uint16 *) p;
        case 3:
            if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
                return p[0] << 16 | p[1] << 8 | p[2];
            else
                return p[0] | p[1] << 8 | p[2] << 16;
        case 4:
            return *(Uint32 *) p;
        default:
            return 0; /* 起こるはずがないが、ワーニングを回避 */
    }
}