#include "Game.h"                   // ゲーム共通
#include "Player.h"                 // プレーヤー相関
#include "Map.h"                    // マップ相関
#include "BackGround.h"             // 背景（星空）相関
#include "Item.h"                   // アイテム相関
#include "KeyState.h"               // キー制御
#include "Info.h"                   // 画面情報相関
#include "FpsControl.h"             // FPS制御
#include "CollisionDetection.h"     // 当たり判定相関

Game::Game() {
    // ランダム初期化
    srand(time(0));
    rand();

    // スクリーン初期化
    sdlInit = SDL_Init(SDL_INIT_VIDEO);

    // タイトル
    SDL_WM_SetCaption("SPACE SHUFFLE", NULL);

    // スクリーン作成
    screen = SDL_SetVideoMode(WIDTH, HEIGHT, VIDEOBIT, SDL_SWSURFACE);

    // 画像読み込み
    image = (SDL_Surface *) IMG_Load("image/material.bmp");
    if (image == NULL) {
        printf("Error: MaterialFile could not be loaded.\n");
    } else {
        printf("OK: MaterialFile was successfully loaded.\n\n");
    }
    // カラーキーの設定
    SDL_SetColorKey(image, SDL_SRCCOLORKEY, 0x00000000);

    // パレットの初期化
    fmt = screen -> format;
    backColor = SDL_MapRGB(fmt, 0x14, 0x14, 0x14);
    white = SDL_MapRGB(fmt, 0xff, 0xff, 0xff);

    endFlg = 0;
}

Game::~Game() {
    // 解放
    SDL_FreeSurface(screen);
    SDL_FreeSurface(image);
    SDL_Quit();
}

int Game::GameInit() {
    /******************************
       レコードファイルの読み込み
     ******************************/
    // レコードファイルの読み込み
    fp = NULL;
    fp = fopen("./Rec.dat", "r");
    if (fp == NULL) {
        newFileFlg = 1;
        recordANum = recordBNum = 0;
    } else {
        newFileFlg = 0;
        printf("OK: RecordFile was successfully loaded.\n");
    }
    printf("\n");
    // レコードを変数に読み込み
    if (fp != NULL) {
        fgets(record, 7, fp);
        recordANum = atoi(&record[0]);
        fgets(record, 7, fp);
        recordBNum = atoi(&record[0]);
        fclose(fp);
        printf("ModeA Rec: %d\nModeB Rec: %d\n\n", recordANum, recordBNum);
    }
    /*************************************
       レコードファイルの読み込み 終わり
     *************************************/

    // エラーチェック
    if (sdlInit < 0) {
        //return -1;
    }
    if (screen == NULL) {
        //return -1;
    }

    // 以下、初期化
    dushFlg = 0;
    dushCount = 0;
    levelupCnt = 0;
    gameOverFlg = 0;
    returnFlg = 0;
    endFlg = 0;

    overSrc.x = 122;
    overSrc.y = 192;
    overSrc.w = 239;
    overSrc.h = 117;
    overDsr.x = WIDTH / 2 - 239 / 2;
    overDsr.y = HEIGHT / 2 - 117 / 2 - 16;
    overAlpha = 0;

    retrySrc.x = 0;
    retrySrc.y = 238;
    retrySrc.w = 97;
    retrySrc.h = 23;
    retryDsr.x = 139;
    retryDsr.y = 229 - 16;

    titleKeySrc.x = 0;
    titleKeySrc.y = 284;
    titleKeySrc.w = 97;
    titleKeySrc.h = 23;
    titleKeyDsr.x = 246;
    titleKeyDsr.y = 229 - 16;

    exitSrc.x = 0;
    exitSrc.y = 261;
    exitSrc.w = 85;
    exitSrc.h = 23;
    exitDsr.x = WIDTH / 2 - 85 / 2;

    titleSrc.x = 0;
    titleSrc.y = 75;
    titleSrc.w = 422;
    titleSrc.h = 117;
    titleDsr.x = WIDTH / 2 - 422 / 2;
    titleDsr.y = HEIGHT / 2 - 117 / 2 - 16 - 6;
    titleAlpha = 0;

    aSrc.x = 0;
    aSrc.y = 192;
    aSrc.w = 105;
    aSrc.h = 23;
    aDsr.x = 130;
    aDsr.y = 207;

    bSrc.x = 0;
    bSrc.y = 215;
    bSrc.w = 105;
    bSrc.h = 23;
    bDsr.x = 245;
    bDsr.y = 207;
}

// タイトル画面
void Game::Title() {
    Player Player;
    Map Map;
    BackGround BG[STARNO];
    Info Info;
    KeyState Key;
    FpsControl Fps;

    titleReturnFlg = 0;

    while (true) {
        // キー判定
        Key.Key(Key.event, &dushFlg, gameOverFlg, &returnFlg, &endFlg, &titleReturnFlg, &mode);

        // 背景の塗りつぶす
        SDL_FillRect(screen, NULL, backColor);
        // 透明度初期化
        SDL_SetAlpha(image, SDL_SRCALPHA, 255);

        // マップのスクロールスピード設定とマップの描画
        Map.speedX = 1;
        Map.ScrollDraw(screen, &Player.pX, &Player.pY, dushFlg);

        // 星空の背景描画&移動
        for (i = 0; i < STARNO; i++) {
            BG[i].Draw(screen, white);
            BG[i].Move(Map.speedX);
        }

        // EXIT座標
        exitDsr.y = 240;

        // タイトルの表示、透明度だんだん上がっていく
        titleAlpha += 5;
        if (titleAlpha >= 255) {
            titleAlpha = 255;
        }
        // 透明度設定
        SDL_SetAlpha(image, SDL_SRCALPHA, titleAlpha);
        // SPACE SHUFFLE
        SDL_BlitSurface(image, &titleSrc, screen, &titleDsr);
        // MODE A
        SDL_BlitSurface(image, &aSrc, screen, &aDsr);
        // MODE B
        SDL_BlitSurface(image, &bSrc, screen, &bDsr);
        // EXIT
        SDL_BlitSurface(image, &exitSrc, screen, &exitDsr);
        // レコード描画
        Info.titleRec(screen, image, recordANum, recordBNum);

        // ダブルバッファ
        SDL_Flip(screen);

        // リターン制御
        if (titleReturnFlg) {
            return;
        }

        // FPS制御
        Fps.FpsWait();
    }
}

void Game::GameMain(int *titleFlg) {
    Player Player[20];
    Map Map;
    BackGround BG[STARNO];
    Item Item[ITEMNO];
    CollisionDetection CD;
    KeyState Key;
    Info Info;
    FpsControl Fps;

    // ゲームループ
    while (!endFlg) {
        // キー判定
        Key.Key(Key.event, &dushFlg, gameOverFlg, &returnFlg, &endFlg, &titleReturnFlg, &mode);

        // ダッシュ相関
        if (dushFlg == 1) {
            Info.dushGaugeLen--;            // ダッシュするときに､ダッシュゲージが減る
            for (i = 0; i < 20; i++) {
                Player[i].Color = 1;        // ダッシュするときに､プレーヤーの色が変わる
            }
            if (Info.dushGaugeLen <= 0) {
                Info.dushGaugeLen = 0;      // ダッシュゲージ最小値
                dushFlg = 0;                // ダッシュゲージが空の場合､ダッシュできない
            }
        } else {
            for (i = 0; i < 20; i++) {
                Player[i].Color = 0;        // プレーヤーの色を戻す
            }
        }

        /**********************
          ダブルバッファー開始
         **********************/
        // 背景の塗りつぶす
        SDL_FillRect(screen, NULL, backColor);
        // 透明度初期化
        SDL_SetAlpha(image, SDL_SRCALPHA, 255);

        /********************
          プレイヤー残像処理
         ********************/
        // 1フレーム前の位置を順次に保存
        for (i = 19; i > 0; i--) {
            Player[i].pX = Player[i - 1].pX;
            Player[i].pY = Player[i - 1].pY;
        }
        // 残像の透明度制御&残像描画
        for (i = 1; i < 20; i++) {
            SDL_SetAlpha(image, SDL_SRCALPHA, (20 - i)*5);
            Player[i].Draw(screen, image, mode);
        }

        // マップの描画&移動
        Map.ScrollDraw(screen, &Player[0].pX, &Player[0].pY, dushFlg);
        Map.Move(screen, Key.keyFlg, Player[0].pX, Player[0].pY, dushFlg, mode);

        // 星空の背景描画&移動
        for (i = 0; i < STARNO; i++) {
            BG[i].Draw(screen, white);
            BG[i].Move(Map.speedX);
        }

        // 透明度初期化
        SDL_SetAlpha(image, SDL_SRCALPHA, 255);

        // アイテム描画&バンド移動
        for (i = 0; i < ITEMNO; i++) {
            Item[i].Draw(screen, image, gameOverFlg, mode);
            Item[i].Move();
        }

        // プレイヤー描画&移動
        SDL_SetAlpha(image, SDL_SRCALPHA, 255);
        Player[0].Draw(screen, image, mode);
        Player[0].Move(screen, Key.keyFlg, dushFlg, mode);

        // ダッシュゲージが自動的に溜まる
        if (!gameOverFlg) {
            // 溜まるスピードの制御、レベルが高ければ高いほど、溜まるスピードが早い
            dushCount++;
            dushJudge = 25 - Map.speedX * 2;
            // 溜まるスピードの限界
            if (dushJudge <= 3) {
                dushJudge = 3;
            }
            if (dushCount >= dushJudge) {
                Info.dushGaugeLen++;
                // ダッシュゲージの限界長さ
                if (Info.dushGaugeLen >= 89) {
                    Info.dushGaugeLen = 89;
                }
                dushCount = 0;
            }
        }

        // レベルアップ処理
        if (!gameOverFlg) {
            levelupCnt++;
            // レベルアップすると、マップのスクロールスピードが上がる
            if (levelupCnt % 1000 == 0) {
                Map.speedX++;
            }
        }


        // プレイヤーとアイテムの当たり判定
        for (i = 0; i < ITEMNO; i++) {
            if (CD.PlayerAndItem(Player[0].pX, Player[0].pY, Item[i].itemDsr.x, Item[i].itemDsr.y) == true && Item[i].itemAlpha == 255) {
                // アイテムに当たったら、ダッシュゲージが溜まる
                Info.dushGaugeLen += 10;
                if (Info.dushGaugeLen >= 89) {
                    Info.dushGaugeLen = 89;
                }
                // アイテムの透明度
                Item[i].itemAlpha = 0;
                // アイテムの位置を再初期化
                Item[i].Init();
            }
        }

        // レコードを超えるときのレコード表示
        if (mode == 0) {
            if (levelupCnt > recordANum) {
                Info.RecDraw(screen, image, levelupCnt);
            }
        }
        if (mode == 1) {
            if (levelupCnt > recordBNum) {
                Info.RecDraw(screen, image, levelupCnt);
            }
        }

        // GAMEOVER判定（左側、上、下）
        if (Player[0].pX <= -PLAYERWIDTH || Map.mDsrA.y > 155 || Map.mDsrA.y<-155 || Map.mDsrB.y > 155 || Map.mDsrB.y<-155) {
            gameOverFlg = 1;
        }
        if (gameOverFlg) {
            // GAMEOVERの場合、プレーヤーを消えさせる
            Player[0].pX -= 10;
            // GAMEOVER情報の透明度
            overAlpha += 5;
            if (overAlpha >= 255) {
                overAlpha = 255;
            }
            // EXIT座標
            exitDsr.y = 246;
            // 透明度の設定
            SDL_SetAlpha(image, SDL_SRCALPHA, overAlpha);
            // GAMEOVER
            SDL_BlitSurface(image, &overSrc, screen, &overDsr);
            // RETRY
            SDL_BlitSurface(image, &retrySrc, screen, &retryDsr);
            // TITLE
            SDL_BlitSurface(image, &titleKeySrc, screen, &titleKeyDsr);
            // EXIT
            SDL_BlitSurface(image, &exitSrc, screen, &exitDsr);
        }

        // 左上の看板描画
        Info.Draw(screen, image, Map.speedX, levelupCnt, mode);

        // ダブルバッファ
        SDL_Flip(screen);

        /**********************
          ダブルバッファー終了
         **********************/
        if (returnFlg) {
            // レコードの書き込み ------------------------
            if (mode == 0) {
                if (levelupCnt > recordANum) {
                    recordANum = levelupCnt;
                }
            }
            if (mode == 1) {
                if (levelupCnt > recordBNum) {
                    recordBNum = levelupCnt;
                }
            }
            fp = NULL;
            if (newFileFlg == 1) {
                fp = fopen("./Rec.dat", "aw");
            } else if (newFileFlg == 0) {
                fp = fopen("./Rec.dat", "w");
            }
            if (fp == NULL) {
                printf("Error: RecordFile could not be loaded.\n");
            } else {
                printf("OK: RecordFile was successfully loaded.\n");
            }
            fprintf(fp, "%d\n%d\n", recordANum, recordBNum);
            fclose(fp);
            // レコードの書き込み 終わり ------------------

            // タイトルに戻る
            if(returnFlg == 2){
                *titleFlg = 1;
            }
            return;
        }

        // FPS制御&表示
        Fps.FpsWait();
        Fps.FpsDisplay();
    }
}