#include "Game.h"
#include "Map.h"
#include "CollisionDetection.h"

Map::Map() {
    /*********************
       マップの読み込み
     *********************/
    // マップデータファイルの読み込み
    fp = NULL;
    fp = fopen("./image/mapImg/map.dat", "r");
    if (fp == NULL) {
        printf("Error: MapDataFile could not be loaded.\n");
    } else {
        printf("OK: MapDataFile was successfully loaded.\n");
    }
    printf("\n");

    for (i = 0; i < MAPNO; i++) {
        // ファイル名を配列に読み込み
        fscanf(fp, "%s", &fileName[0]);
        // マップファイルを読み込み
        mImg[i] = (SDL_Surface *) IMG_Load(&fileName[0]);
        if (mImg[i] == NULL) {
            printf("Error: [%s] could not be loaded.\n", fileName);
        } else {
            printf("OK: MapFile [%s] was successfully loaded.\n", fileName);
        }
        // カラーキーの設定
        SDL_SetColorKey(mImg[i], SDL_SRCCOLORKEY, 0x141414);
    }
    fclose(fp);
    printf("_________________________________________________________________\n");

    /****************************
       マップの読み込み 終わり
     ****************************/

    // 以下、初期化
    mSrcA.x = 0;
    mSrcA.y = 0;
    mSrcA.w = WIDTH;
    mSrcA.h = HEIGHT;
    mDsrA.x = 0;
    mDsrA.y = 0;

    mSrcB.x = 0;
    mSrcB.y = 0;
    mSrcB.w = WIDTH;
    mSrcB.h = HEIGHT;
    mDsrB.x = 0;
    mDsrB.y = 0;

    mapNoA = 0;
    mapNoB = 0;
    scrollX = 0;

    changeFlg = 0;

    speedX = 1;
    speedY = 5;

    vY = 0;
}

Map::~Map() {
    // 解放
    for (i = 0; i < MAPNO; i++) {
        SDL_FreeSurface(mImg[i]);
    }
}

void Map::ScrollDraw(SDL_Surface *screen, int *pX, int *pY, int dushFlg) {
    CollisionDetection CD;
    // ランダムマップ生成
    if (scrollX >= WIDTH) {
        scrollX = 0;
        mapNoA = mapNoB;
    }
    if (scrollX == 0) {
        changeFlg = 1;
    }
    if (changeFlg == 1) {
        while (true) {
            mapNoB = rand() % MAPNO;
            if (mapNoA == mapNoB) {
                continue;
            } else {
                changeFlg = 0;
                break;
            }
        }
    }
    mDsrA.x = 0 - scrollX;
    mDsrB.x = WIDTH - scrollX;

    // マップ描画
    SDL_BlitSurface(mImg[mapNoA], &mSrcA, screen, &mDsrA);
    SDL_BlitSurface(mImg[mapNoB], &mSrcB, screen, &mDsrB);

    // スクロールスピード
    scrollX += speedX;

    // ダッシュしていない場合のマップとの当たり判定
    if (dushFlg == 0) {
        for (i = 0; i < speedX; i++) {
            if (!CD.PlayerAndMap(screen, *pX, *pY, RIGHT)) {
                *pX -= speedX;
                break;
            }
        }
    }

    // モードBのマップ移動用
    mDsrA.y = vY;
    mDsrB.y = vY;
}

void Map::Move(SDL_Surface *screen, unsigned char keyFlg, int pX, int pY, int dushFlg, int mode) {
    CollisionDetection CD;
    // モードBのマップ移動と当たり判定
    if (mode == 1) {
        if (dushFlg == 0) {
            if (keyFlg & 0x01) {
                for (i = 0; i < speedY; i++) {
                    if (CD.PlayerAndMap(screen, pX, pY, UP)) {
                        vY++;
                    } else {
                        vY--;
                    }
                }
            }

            if (keyFlg & 0x02) {
                for (i = 0; i < speedY; i++) {
                    if (CD.PlayerAndMap(screen, pX, pY, DOWN)) {
                        vY--;
                    } else {
                        vY++;
                    }
                }
            }
        }
    }
}