#include "Game.h"
#include "Player.h"
#include "CollisionDetection.h"

Player::Player() {
    mySrc.w = PLAYERWIDTH;
    mySrc.h = PLAYERHEIGHT;

    pX = 0;
    pY = HEIGHT / 2 - mySrc.h / 2;

    Speed = 5;
    Color = 0;
}

void Player::Draw(SDL_Surface *screen, SDL_Surface *image, int mode) {
    // モードによって、ダッシュするときのプレイヤーの色が違う
    if (mode == 0) {
        mySrc.x = 0;
        mySrc.y = 12 * Color;
    } else if (mode == 1) {
        if (Color == 0) {
            mySrc.x = 0;
            mySrc.y = 0;
        } else if (Color == 1) {
            mySrc.x = 64;
            mySrc.y = 24;
        }
    }

    myDsr.x = pX;
    myDsr.y = pY;

    // プレーヤー描画
    SDL_BlitSurface(image, &mySrc, screen, &myDsr);
}

void Player::Move(SDL_Surface *screen, unsigned char keyFlg, int dushFlg, int mode) {
    CollisionDetection CD;
    int i;
    
    //モードA、上下キーでプレーヤーを動かす
    if (mode == 0) {
        if (keyFlg & 0x01) {
            for (i = 0; i < Speed; i++) {
                if (CD.PlayerAndMap(screen, pX, pY, UP)) {
                    pY--;
                } else {
                    break;
                }
            }
        }
        if (keyFlg & 0x02) {
            for (i = 0; i < Speed; i++) {
                if (CD.PlayerAndMap(screen, pX, pY, DOWN)) {
                    pY++;
                } else {
                    break;
                }
            }
        }
    }
    
    // 共通左右移動
    if (keyFlg & 0x04) {
        if (dushFlg == 0) {
            for (i = 0; i < Speed; i++) {
                if (CD.PlayerAndMap(screen, pX, pY, LEFT)) {
                    pX--;
                } else {
                    break;
                }
            }
        } else if (dushFlg == 1) {
            pX -= Speed * 2;
        }
    }
    if (keyFlg & 0x08) {
        if (dushFlg == 0) {
            for (i = 0; i < Speed; i++) {
                if (CD.PlayerAndMap(screen, pX, pY, RIGHT)) {
                    pX++;
                } else {
                    break;
                }
            }
        } else if (dushFlg == 1) {
            pX += Speed * 2;
        }
    }

    // 右チェック（左チェックなし、左から出たらGAMEOVERになるため）
    if (pX >= WIDTH - PLAYERWIDTH - 1) {
        // 色判定微調整、-1しないとマップの中に掘り込んでしまう
        pX = WIDTH - PLAYERWIDTH - 1;
    }
}

